@@ -59,6 +59,9 @@ import java.util.*;
 import freemarker.ext.beans.BeansWrapper;
 import freemarker.log.Logger;
 import freemarker.template.*;
+import freemarker.template.utility.DateUtil;
+import freemarker.template.utility.DateUtil.DateToISO8601CalendarFactory;
+import freemarker.template.utility.StringUtil;
 import freemarker.template.utility.UndeclaredThrowableException;
 
 /**
@@ -110,6 +113,12 @@ public final class Environment extends Configurable {
     private DateFormat timeFormat, dateFormat, dateTimeFormat;
     private Map[] dateFormats;
     private NumberFormat cNumberFormat;
+    
+    /**
+     * Used by the "iso_" built-ins to accelerate formatting.
+     * @see #getISOBuiltInCalendar() 
+     */
+    private DateToISO8601CalendarFactory isoBuiltInCalendarFactory;
 
     private Collator collator;
 
@@ -188,8 +197,10 @@ public final class Environment extends Configurable {
             try {
                 doAutoImportsAndIncludes(this);
                 visit(getTemplate().getRootTreeNode());
-                // Do not flush if there was an exception.
-                out.flush();
+                // It's here as we must not flush if there was an exception.
+                if (getAutoFlush()) {
+                    out.flush();
+                }
             } finally {
                 // It's just to allow the GC to free memory...
                 clearCachedValues();
@@ -356,7 +367,7 @@ public final class Environment extends Configurable {
          if (thrownException != null) {
              if (attemptLogger.isDebugEnabled()) {
                  attemptLogger.debug("Error in attempt block " + 
-                         attemptBlock.getStartLocation(), thrownException);
+                         attemptBlock.getStartLocationQuoted(), thrownException);
              }
              try {
                  recoveredErrorStack.add(thrownException.getMessage());
@@ -662,7 +673,7 @@ public final class Environment extends Configurable {
 
         // Log the exception
         if(logger.isErrorEnabled()) {
-            logger.error(te.getMessage(), te);
+            logger.error("Template processing error: " + StringUtil.jQuoteNoXSS(te.getMessage()), te);
         }
 
         // Stop exception is not passed to the handler, but
@@ -973,6 +984,20 @@ public final class Environment extends Configurable {
         }
         return -1;
     }
+    
+
+    /**
+     * Returns the {@link DateUtil.DateToISO8601CalendarFactory} used by the
+     * the "iso_" built-ins. Be careful when using this; it should only by used
+     * with {@link DateUtil#dateToISO8601String(Date, boolean, boolean, boolean,
+     * int, TimeZone, DateToISO8601CalendarFactory)}.
+     */
+    DateToISO8601CalendarFactory getISOBuiltInCalendar() {
+        if (isoBuiltInCalendarFactory == null) {
+            isoBuiltInCalendarFactory = new DateUtil.TrivialDateToISO8601CalendarFactory();
+        }
+        return isoBuiltInCalendarFactory;
+    }
 
     /**
      * Returns the {@link NumberFormat} used for the <tt>c</tt> built-in.
