@@ -52,12 +52,40 @@
 
 package freemarker.template;
 
-import java.io.*;
-import java.util.*;
+import java.io.File;
+import java.io.FileNotFoundException;
+import java.io.IOException;
+import java.io.InputStream;
+import java.util.ArrayList;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Locale;
+import java.util.Map;
+import java.util.Properties;
+import java.util.Set;
+import java.util.SortedMap;
+import java.util.TreeMap;
 
-import freemarker.cache.*;
-import freemarker.core.*;
-import freemarker.template.utility.*;
+import freemarker.cache.CacheStorage;
+import freemarker.cache.ClassTemplateLoader;
+import freemarker.cache.FileTemplateLoader;
+import freemarker.cache.MruCacheStorage;
+import freemarker.cache.SoftCacheStorage;
+import freemarker.cache.TemplateCache;
+import freemarker.cache.TemplateLoader;
+import freemarker.core.Configurable;
+import freemarker.core.Environment;
+import freemarker.core.ParseException;
+import freemarker.template.utility.CaptureOutput;
+import freemarker.template.utility.ClassUtil;
+import freemarker.template.utility.HtmlEscape;
+import freemarker.template.utility.NormalizeNewlines;
+import freemarker.template.utility.SecurityUtilities;
+import freemarker.template.utility.StandardCompress;
+import freemarker.template.utility.StringUtil;
+import freemarker.template.utility.XmlEscape;
 
 /**
  * Main entry point into the FreeMarker API, this class encapsulates the 
@@ -68,7 +96,7 @@ import freemarker.template.utility.*;
  * template loading mechanism by using the {@link #setTemplateLoader(TemplateLoader)}
  * method.
  *
- * This object is <em>not synchronized</em>. Thus, the settings must not be changed
+ * <p>This object is <em>not synchronized</em>. Thus, the settings must not be changed
  * after you have started to access the object from multiple threads. If you use multiple
  * threads, set everything directly after you have instantiated the <code>Configuration</code>
  * object, and don't change the settings anymore.
@@ -88,14 +116,19 @@ public class Configuration extends Configurable implements Cloneable {
     public static final String AUTO_IMPORT_KEY = "auto_import";
     public static final String AUTO_INCLUDE_KEY = "auto_include";
     public static final String TAG_SYNTAX_KEY = "tag_syntax";
+    public static final String INCOMPATIBLE_ENHANCEMENTS = "incompatible_enhancements";
     public static final int AUTO_DETECT_TAG_SYNTAX = 0;
     public static final int ANGLE_BRACKET_TAG_SYNTAX = 1;
     public static final int SQUARE_BRACKET_TAG_SYNTAX = 2;
-
-
+    
+    public static final String DEFAULT_INCOMPATIBLE_ENHANCEMENTS = "2.3.0"; // [2.4] 
+    public static final int PARSED_DEFAULT_INCOMPATIBLE_ENHANCEMENTS = StringUtil.versionStringToInt(DEFAULT_INCOMPATIBLE_ENHANCEMENTS); 
+    
     private static Configuration defaultConfig = new Configuration();
     private static String cachedVersion;
     private boolean strictSyntax = true, localizedLookup = true, whitespaceStripping = true;
+    private String incompatibleEnhancements = DEFAULT_INCOMPATIBLE_ENHANCEMENTS;
+    private int parsedIncompatibleEnhancements = PARSED_DEFAULT_INCOMPATIBLE_ENHANCEMENTS;
     private int tagSyntax = ANGLE_BRACKET_TAG_SYNTAX;
 
     private TemplateCache cache;
@@ -288,6 +321,7 @@ public class Configuration extends Configurable implements Cloneable {
         cache.setConfiguration(this);
         cache.setLocalizedLookup(localizedLookup);
     }
+    
     /**
      * @return the template loader that is used to look up and load templates.
      * @see #setTemplateLoader
@@ -307,7 +341,7 @@ public class Configuration extends Configurable implements Cloneable {
     public synchronized void setCacheStorage(CacheStorage storage) {
         createTemplateCache(cache.getTemplateLoader(), storage);
     }
-    
+
     /**
      * Set the explicit directory from which to load templates.
      */
@@ -392,6 +426,57 @@ public class Configuration extends Configurable implements Cloneable {
         return strictSyntax;
     }
 
+    /**
+     * Sets which of the <em>slightly</em> non-backward compatible
+     * bugfixes/enhancements should be enabled. The setting value is the
+     * FreeMarker release version number where the enhancements
+     * to enable were already present. The default is 2.3.0 in 2.3.x, 2.4.0 on
+     * 2.4.x, and so on, thus by default compatibility with the latest x.y.0
+     * release is kept with new releases. If you develop a new application with,
+     * for example, 2.3.25 and you need the non-backward compatible enhancements,
+     * you should set this to 2.3.25. Thus if you later update FreeMarker to a
+     * higher 2.3.x version, you will still only have the incompatible changes
+     * that you have tested your application with.
+     * 
+     * <p>This setting doesn't affect non-backward
+     * compatible security fixes; they are always enabled. This setting also
+     * doesn't affect enhancements where there's a significant chance of
+     * breaking existing applications.
+     * 
+     * <p>Using this setting is a good way of preparing for the next minor
+     * (2nd) version number increase. When that happens, not only the default
+     * value of this setting changes, but it's possible that older values become
+     * unsupported.
+     * 
+     * <p>Currently affected fixes/enhancements:
+     * <ul>
+     *   <li>
+     *     2.3.19: Bug fix: Wrong # tags were printed as static text instead of
+     *     causing parsing error if there was no correct # tag earlier in the
+     *     same template.
+     *   </li>
+     * </ul>
+     *
+     * @since 2.3.19
+     */
+    public void setIncompatibleEnhancements(String version) {
+        parsedIncompatibleEnhancements = StringUtil.versionStringToInt(version);
+        incompatibleEnhancements = version;
+    }
+
+    public String getIncompatibleEnhancements() {
+        return incompatibleEnhancements;
+    }
+    
+    /**
+     * Same as {@link #getIncompatibleEnhancements()}, but returns the version
+     * as an <tt>int</tt>, according to
+     * {@link StringUtil#versionStringToInt(String)}. 
+     */
+    public int getParsedIncompatibleEnhancements() {
+        return parsedIncompatibleEnhancements;
+    }
+    
     /**
      * Sets whether the FTL parser will try to remove
      * superfluous white-space around certain FTL tags.
@@ -652,6 +737,50 @@ public class Configuration extends Configurable implements Cloneable {
         cache.clear();
     }
     
+    /**
+     * Equivalent to <tt>removeTemplateFromCache(name, thisCfg.getLocale(), thisCfg.getEncoding(thisCfg.getLocale()), true)</tt>.
+     */
+    public void removeTemplateFromCache(String name) throws IOException {
+        Locale loc = getLocale();
+        removeTemplateFromCache(name, loc, getEncoding(loc), true);
+    }
+
+    /**
+     * Equivalent to <tt>removeTemplateFromCache(name, locale, thisCfg.getEncoding(locale), true)</tt>.
+     */
+    public void removeTemplateFromCache(String name, Locale locale) throws IOException {
+        removeTemplateFromCache(name, locale, getEncoding(locale), true);
+    }
+
+    /**
+     * Equivalent to <tt>removeTemplateFromCache(name, thisCfg.getLocale(), encoding, true)</tt>.
+     */
+    public void removeTemplateFromCache(String name, String encoding) throws IOException {
+        removeTemplateFromCache(name, getLocale(), encoding, true);
+    }
+
+    /**
+     * Equivalent to <tt>removeTemplateFromCache(name, locale, encoding, true)</tt>.
+     */
+    public void removeTemplateFromCache(String name, Locale locale, String encoding) throws IOException {
+        removeTemplateFromCache(name, locale, encoding, true);
+    }
+    
+    /**
+     * Removes a template from the template cache, hence forcing the re-loading
+     * of it when it's next time requested. This is to give the application
+     * finer control over cache updating than {@link #setTemplateUpdateDelay(int)}
+     * alone does.
+     * 
+     * For the meaning of the parameters, see
+     * {@link #getTemplate(String, Locale, String, boolean)}.
+     */
+    public void removeTemplateFromCache(
+            String name, Locale locale, String encoding, boolean parse)
+    throws IOException {
+        cache.removeTemplate(name, locale, encoding, parse);
+    }    
+    
     /**
      * Returns if localized template lookup is enabled or not.
      * This method is thread-safe and can be called while the engine works.
@@ -716,6 +845,9 @@ public class Configuration extends Configurable implements Cloneable {
      *   <li><code>"tag_syntax"</code>: Must be one of:
      *       <code>"auto_detect"</code>, <code>"angle_bracket"</code>,
      *       <code>"square_bracket"</code>.
+     *   <li><code>"incompatible_enhancements"</code>: The FreeMarker version
+     *       number where the desired enhancements were already implemented.
+     *       See: {@link #setIncompatibleEnhancements(String)}.
      * </ul>
      *
      * @param key the name of the setting.
@@ -789,6 +921,8 @@ public class Configuration extends Configurable implements Cloneable {
                 } else {
                     throw invalidSettingValueException(key, value);
                 }
+            } else if (INCOMPATIBLE_ENHANCEMENTS.equals(key)) {
+                setIncompatibleEnhancements(value);
             } else {
                 callSuper = true;
             }
@@ -804,7 +938,6 @@ public class Configuration extends Configurable implements Cloneable {
         }
     }
     
-    
     /**
      * Add an auto-imported template.
      * The importing will happen at the top of any template that
