@@ -52,11 +52,32 @@
 
 package freemarker.template;
 
-import java.io.*;
-import java.util.*;
-import javax.swing.tree.TreePath;
-import freemarker.core.*;
+import java.io.BufferedReader;
+import java.io.FilterReader;
+import java.io.IOException;
+import java.io.PrintStream;
+import java.io.Reader;
+import java.io.StringWriter;
+import java.io.Writer;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.Vector;
 
+import javax.swing.tree.TreePath;
+
+import freemarker.core.Configurable;
+import freemarker.core.Environment;
+import freemarker.core.FMParser;
+import freemarker.core.LibraryLoad;
+import freemarker.core.Macro;
+import freemarker.core.ParseException;
+import freemarker.core.TemplateElement;
+import freemarker.core.TextBlock;
+import freemarker.core.TokenMgrError;
 import freemarker.debug.impl.DebuggerService;
 
 /**
@@ -145,7 +166,8 @@ public class Template extends Configurable {
                 FMParser parser = new FMParser(this, ltb,
                         getConfiguration().getStrictSyntaxMode(),
                         getConfiguration().getWhitespaceStripping(),
-                        getConfiguration().getTagSyntax());
+                        getConfiguration().getTagSyntax(),
+                        getConfiguration().getParsedIncompatibleEnhancements());
                 this.rootElement = parser.Root();
             }
             catch (TokenMgrError exc) {
