@@ -107,6 +107,7 @@ import freemarker.testcase.models.BooleanHash2;
 import freemarker.testcase.models.BooleanList1;
 import freemarker.testcase.models.BooleanList2;
 import freemarker.testcase.models.MultiModel1;
+import freemarker.testcase.models.VarArgTestModel;
 
 
 public class TemplateTestCase extends TestCase {
@@ -369,6 +370,10 @@ public class TemplateTestCase extends TestCase {
           dataModel.put("bigInteger", new BigInteger("1305575275540"));
           dataModel.put("bigDecimal", new BigDecimal("1305575275539.5"));
         }
+        
+        else if (testName.equals("test-varargs")) {
+          dataModel.put("m", new VarArgTestModel());
+        }
     }
     
     public void runTest() {
